/* -----------------------------------------------------------	*/
/* Reflection Tutorial											*/
/* -----------------------------------------------------------	*/
/* by n0rmAnfreAK												*/
/* visit www.normanfreak.de.vu for updates						*/
/*																*/
/* feel free to email me any critic and comments				*/
/* ------------------------------------------------------------ */

#include <windows.h>
#include <gl\glut.h>
#include <stdio.h>
#include <math.h>

int frames;
long time, lastTime, animationTime;
char title[100];
int mode;
float angleh, anglev;

#define REFLECTION_OFF 0;
#define REFLECTION_SHARP 1;
#define REFLECTION_FUZZY 2;

// all code that has to be run once before the rendering
void Init()	
{
	glClearColor(0.33f,0.225f,0.0f,0.0f);
	glClearStencil(0);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	glEnable(GL_DEPTH_TEST);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);	
	glShadeModel(GL_SMOOTH);
	//glEnable(GL_LIGHTING);
	glEnable(GL_CULL_FACE);
	glClearDepth(1.0);
	mode = REFLECTION_SHARP;

	angleh = anglev = 0.f;

	frames = 0;
	time = 0;
	lastTime = animationTime = GetTickCount();	
}

void drawObjects()
{
	glPushMatrix();
	glTranslatef(0.3, -.2, -1.2);
	glColor4f(.9, 0., 0.2, 1.);
	glutSolidSphere(.4, 10, 10);
	glPopMatrix();
}

void drawMirror()
{
	// mirrorface
	glColor4f(.8, .8, .9, .7);
	glBegin(GL_QUADS);
	glVertex3f(-1, -1, 0);
	glVertex3f(-1, 1, 0);
	glVertex3f(1, 1, 0);
	glVertex3f(1, -1, 0);
	glEnd();
}

void Scene()
{
	
	glLoadIdentity ();
	gluLookAt(0, 3, -3, 0, 0, 0, 0, 1, 0); 

	// rotate by mousemovement
	glRotatef(anglev, 1, 0,	0);
	glRotatef(angleh, 0, 1, 0);
	
	// draw the regular objects in the scene
	drawObjects();
	
	if(mode > 0)
	{
		// clear the stencil buffer
		glClear(GL_STENCIL_BUFFER_BIT);

		// turn off visible rendering 
		glColorMask( GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);

		// don't override depthvalues
		glDepthMask( GL_FALSE);

		// use the mirror face as a stencilmask
		glStencilFunc(GL_ALWAYS, ~0, 0);
		glEnable(GL_STENCIL_TEST);
		glStencilOp(GL_KEEP, GL_KEEP, GL_INCR);
		drawMirror();

		// turn on visible rendering
		glColorMask( GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
		glDepthMask( GL_TRUE);

		// draw the objects again behind the mirror
		glScalef(1,1,-1);

		// only if we see "through" the mirror
		glStencilFunc(GL_EQUAL, 1, 0xff);
		drawObjects();	
		glDisable(GL_STENCIL_TEST);
		
	}
	// draw the mirror itself (must be done last cause it's transparent)
	drawMirror();
}


void RenderScene(void)
{
	// clear all buffers
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
	
	// render Scene ambient
	Scene();

	glutSwapBuffers (); 
	glFlush ();

	// bruteforce to get fps-count
	frames++;
	long time = GetTickCount();
	if(time - lastTime > 1000)
	{
		lastTime = time;
		sprintf(title, "Reflection Tutorial  v0.3 - fps : %i", frames);
		glutSetWindowTitle(title);
		frames = 0;
	}
}

// all code to reshape the window
void Reshape(int width,int height)
{	
	glMatrixMode(GL_PROJECTION); 
	glLoadIdentity();
	glViewport(0,0,width,height);
	
	GLfloat fAspect;
	if(height==0)
		height=1;
	fAspect=(float)width/(float)height;

	// clipping range !!!! WICHTIG ;)
	gluPerspective(60.0f, fAspect, 0.1, 300.0);
	glMatrixMode(GL_MODELVIEW); 
}

// all code to animate the scene is placed here

void Animate ()
{
	
	RenderScene();
}


void Keyboard(unsigned char key, int x, int y)
{
	// perform keypressed here
	switch(key)
	{
	case '1':
		mode = REFLECTION_OFF;
		break;

	case '2':
		mode = REFLECTION_SHARP;
		break;

	case '3':
		mode = REFLECTION_FUZZY;
		break;
	}
}

int oldX, oldY;
void MouseLook(int x, int y)
{
	angleh += (oldX - x)/2.0f;
	anglev += (oldY - y)/2.0f;
	RenderScene();
	oldX = x;
	oldY = y;
}

void MousePressed(int button, int state, int x, int y)
{
	if(button == GLUT_LEFT_BUTTON)
	{	
		oldX = x;
		oldY = y;
	}
}

int main(int argc, char **argv)
{
	printf ("Reflection Tutorial \n\n");
	glutInit ( &argc, argv );
	glutInitDisplayMode ( GLUT_DOUBLE|GLUT_RGBA|GLUT_DEPTH|GLUT_STENCIL );
	glutInitWindowSize ( 600,600 );	
	glutCreateWindow ("Reflection Tutorial v0.3");
	glutDisplayFunc ( RenderScene );
	glutReshapeFunc ( Reshape );
	glutIdleFunc ( Animate );
	glutKeyboardFunc ( Keyboard );
	glutMouseFunc ( MousePressed );
	glutMotionFunc ( MouseLook );
	Init();
	glutMainLoop ();
	return 0;
}
